;	DATE: 19/12/2009
;
;	D.G. MORTIMER
;
;	Programmable i2c 10 bit a-d converter
;
;	
;	
;
;	WRITTEN FOR PIC 16F88
;
;	Last update - 26/10/2012
;
;	reason for change: extend to 7 cnannels
;
;........................................................................

	#include p16f88.inc	;pic variables
	list p=p16f88		;pic parameters

	uSec10 equ h'20'		;delay variables
	ms1 equ h'21'
	sec1 equ h'22'
	ms10 equ h'23'

	adlo equ h'24'
	adhi equ h'25'
	adcpin equ h'26'			;temp storage of current adc pin

	an0 equ b'00000000'
	an1 equ b'00001000'
	an2 equ b'00010000'
	an3 equ b'00011000'
	an4 equ b'00100000'
	an5 equ b'00101000'
	an6 equ b'00110000'

	delay_byte equ h'27'
	databyte equ h'28' 		;register for data to o/p or i/p
	buff_size equ 8		;number of internal addresses
	offset equ h'2a'		;internal address pointer
	addr0 equ h'2b'		;internal memory locations
	addr1 equ h'2c'		;more or less as required or ram permits
	addr2 equ h'2d'
	addr3 equ h'2e'
	addr4 equ h'2f'
	addr5 equ h'30'
	addr6 equ h'31'
	addr7 equ h'32'

	tempw equ h'70'
	status_temp equ h'71'
	fsr_temp equ h'72'

;===============================================


	address equ b'00011000'	;base address of slave


	__config _config1, _lvp_off & _boden_off & _mclr_off & _wdt_off & _intrc_io & _cp_off

	ORG 0		;LOCATION OF RESET VECTOR
	GOTO START	

	ORG  4		;LOCATION OF PERIPHERAL INTERRUPT VECTOR 
	GOTO INTERRUPT	;INTERRUPT ROUTINE LOCATION

	ORG 6		;LOCATION OF RESET START PROGRAMME

start:
	clrf porta		;get rid of any leftover data
	clrf portb
	bcf status,irp
	bcf status,rp1
	bsf status,rp0		;bank 1
	movlw 255
	movwf ansel		;all a-d channels
	clrf adcon1
	bsf adcon1,7		;vref set to vdd vss
	movlw h'ff'
	movwf trisa		;port a inputs
	movwf trisb		;port b inputs
;	bsf trisb,1		;scl and sda as inputs for i2c
;	bsf trisb,4		;to be doubly sure
	movlw b'01100000'	;set int rc osc to 4MHz
	movwf osccon
	clrf sspstat		;clear ssp status register
	bcf status,rp0		;select bank 0
	movf portb,w		;get value of port b pins
	andlw b'00001100'	;mask for address bits
	movwf tempw		;save bits in temp register
	bcf status,c		;clear carry bit
	rrf tempw,w		;shit bits to pos 1 and 2
	iorlw address		;combine with base address
	bsf status,rp0		;bank 1
	movwf sspadd		;put the calculated address in i2c function
	bsf pie1,sspie		;enable interrupts
	bcf status,rp0		;bank 0
	bcf pir1,sspif
	movlw b'00110110'	;set up for slave read no interrupt
	movwf sspcon
	bsf intcon,peie		
	bsf sspcon,sspen
	bsf intcon,gie		;switch on global interrupts
	bcf adcon0,adon		;switch off a-d module
;===================================================================
;                    end of pic setup routine
;  		beginning of main-loop initialisation
;===================================================================
	clrf offset		;set up indirect address for slave regs

;===================================================================
;===================================================================
;       main slave function programme starts here
;===================================================================
;===================================================================

	clrf addr0	
	clrf addr1
	clrf addr2
	
main_loop:

	movf addr2,w		;wait for start convert signal signal (addr2>0)
	sublw 0
	btfsc status,z
	goto main_loop
	movf addr2,w		;check for valid cannel number
	sublw 7
	btfss status,c		;channel <= 7 (c=1 result positive)
	goto main_loop		;invalid channel number so no conversion
	movlw 1
	subwf addr2,f		;convert channel number to adcon code
	bcf status,c		;pre clear carry bit
	rlf addr2,f		;shift channel bits to correct position
	rlf addr2,f
	rlf addr2,f
	movf addr2,w		;get new channel code and start conversion
	call ad_convert		;get adc value
	clrf addr2		;clear control byte so no more conversions done
	bcf adcon0,adon
	goto main_loop

ad_convert:
	bsf adcon0,adon
	movwf adcpin		;save channel number
	movlw b'11000111'	;clear channel bits in adcon0
	andwf adcon0,w
	iorwf adcpin,w		;set up new channel bits
	movwf adcon0
	bsf adcon0,go_done		;start conversion

wait_done:
	btfsc adcon0,go_done	;wait for conversion to complete
	goto wait_done
;	movlw 3			;wait for system to settle
;	call delay10		;30 uSec
	movf adresh,w		;put ad result hi byte in bank0 ram
	movwf addr0
	bsf status,rp0		;get ad result lo byte
	movf adresl,w	
	bcf status,rp0
	movwf addr1		;and put it in bank0 ram 
	return


	




;===================================================================

bank0:
	bcf status,rp0
	return

bank1:
	bsf status,rp0
	return
;
;
;===================================================================
delay10:			;10uSec per loop
	movwf usec10		;come in with uSx10 count in w reg
delay10a:
	nop			;pad out to 10uSecs
	nop
	nop
	nop
	nop
	nop
	decfsz usec10,f
	goto delay10a		;loop count 10uSec loops
	return

delayms:
	movwf ms1		;come in with mSec count in W reg
delaymsa:
	movlw 100		;set up for 100 x 10uSec = 1 mSec
	call delay10
	decfsz ms1,f
	goto delaymsa
	return

delayms10:			;come in with mSx10 count in W reg
	movwf ms10
delayms10a:
	movlw 10
	call delayms
	decfsz ms10,f
	goto delayms10a
	return

delaysec:
	movwf sec1		;come in with seconds count in W reg
delayseca:
	movlw 100		;1000 mSec
	call delayms10			
	decfsz sec1,f
	goto delayseca
	return

;========================================================================
;	End of Main Loop Routines etc.
;	Beginning of interrupt routines
;========================================================================
;++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
;========================================================================
;	this is the data i/o section - i2c slave main routine
;========================================================================
;++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++

interrupt:
	movwf tempw		;save contents of w reg
	swapf status,w
	movwf status_temp	;save contents of status reg
	movf fsr,w
	movwf fsr_temp		;save fsr register
	clrf status		;bank 0
	bcf pir1,sspif

	movf sspbuf,w		;clear data buffer 
	movwf databyte		;save in databyte
	bsf status,rp0		;bank 1
	btfsc sspstat,d_a	;check if data or address
	goto data_mode

;========================================================================
;	get and process mode byte	
;========================================================================

	btfsc sspstat,r_w	;check if we need a mode byte
	goto send
	call inbyte		;read mode byte
	movlw buff_size		;get number of internal addresses
	subwf databyte,w	;check if valid address
	btfsc status,c		;valid if carry is 0 
	clrf databyte		;if bad address, assume address 0
	movf databyte,w		;good address, so put it in <offset>
	movwf offset
	goto int_end

data_mode:

	btfss sspstat,r_w	;check if read or write
	goto receive		;input data
send:
	bcf status,rp0		;bank 0
	movlw addr0		;physical location of address block	
	addwf offset,w		;point to required address
	movwf fsr		;put address in indirect address mode
	movf indf,w		;get data to be output
	btfsc sspcon,ckp	;check if another byte
	goto int_end		;no so finish
	movwf sspbuf		;send data to master
	nop			;delay needed to correct timing
	nop
	nop
	nop
	nop
	bsf sspcon,ckp
	bcf pir1,sspif
	incf offset,f
	movlw buff_size
	subwf offset,w
	btfsc status,z
	clrf offset
	goto int_end

receive:
	bcf status,rp0		;bank 0
	movlw addr0		;physical location of address block
	addwf offset,w		;point to required address
	movwf fsr		;put address in indirect address mode
	movf databyte,w
	movwf indf		;put data in appropriate address
	incf offset,f		;automatic increment of address
	movlw buff_size
	subwf offset,w		;check if exceeded address limit
	btfsc status,z
	clrf offset		;back to start of addresses
	goto int_end	

;========================================================================

int_end:

	bcf status,rp0		;bank 0
	movf fsr_temp,w
	movwf fsr
	bcf pir1,sspif		;clear the interrupt flag		
	swapf status_temp,w	;restore temps
	movwf status
	swapf tempw,f
	swapf tempw,w

	retfie  	

inbyte:
	bcf status,rp0		;bank 0
	btfss pir1,sspif	;wait for databyte in
	goto inbyte
	movf sspbuf,w		;read data buffer and put in databyte reg
	movwf databyte
	bcf pir1,sspif		;clear interrupt flag
	return	


        end		;END OF PROGRAMME - MANDATORY COMMAND
